/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * JBIListener.java - Initial Developper : ofabre
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.engine.xslt.listener;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.logging.Level;

import javax.activation.DataHandler;
import javax.jbi.messaging.MessagingException;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import org.objectweb.petals.component.framework.BusinessException;
import org.objectweb.petals.component.framework.listener.AbstractJBIListener;
import org.objectweb.petals.component.framework.su.ServiceUnitManager;
import org.objectweb.petals.component.framework.util.ClassLoaderUtil;
import org.objectweb.petals.component.framework.util.Exchange;
import org.objectweb.petals.component.framework.util.Extensions;
import org.objectweb.petals.component.framework.util.StringHelper;
import org.objectweb.petals.engine.xslt.StreamDataSource;
import org.objectweb.petals.engine.xslt.Xslt;
import org.objectweb.petals.engine.xslt.XsltImpl;

/**
 * The xslt binding component JBIListener
 *
 * @author alouis, ofabre - EBM Websourcing
 *
 */
public class JBIListener extends AbstractJBIListener {

    private Xslt xslt;

    private final static String XSL = "xsl";
    private final static String OUTPUT_ATTACHMENT_NAME= "output-attachment-name";

    @Override
    protected void init() {
        xslt = new XsltImpl();
    }

    @Override
    protected void onJBIMessage(Exchange exchange) {
    	getLogger().log(Level.INFO,"A new exchange is received.");
		
        try {
            if (exchange.isInOutPattern() || exchange.isInOptionalOutPattern()) {
                processExchange(exchange);
            } else {
                exchange.setError(new Exception("Exchange pattern ("
                        + exchange.getPattern() + ") not supported"));
            }
        } catch (BusinessException e1) {
            try {
                exchange.setFault(e1);
            } catch (MessagingException e) {
                getLogger().log(Level.SEVERE, "Can't return fault to consumer",
                        e);
            }
        } catch (MessagingException e) {
            exchange.setError(e);
        }
    }

    /**
     * Process the transformation and set the response as Source or attachment
     * @param exchange
     * @throws BusinessException
     * @throws MessagingException
     */
    private void processExchange(Exchange exchange) throws BusinessException, MessagingException{
    	getLogger().log(Level.INFO,"Process the transformation.");
    	
    	// Get in Message content
        Source inContent = exchange.getInMessageContentAsSource();

    	Extensions extensions = new Extensions(getProvides()
                .getExtensions());
    	
    	// get the XSL
    	InputStream xsl = getXslInputStream(exchange, extensions);
    	if(xsl == null){
    		exchange.setError(new Exception("No XSL style sheet is found to transform the document."));
    	}
    	
    	// transform the input
        ByteArrayOutputStream outContent = this.xslt.transform(inContent,xsl);
        
        String outputAttachmentName = extensions.getPetalsExtensions().getProperty(OUTPUT_ATTACHMENT_NAME);
        if(StringHelper.isNullOrEmpty(outputAttachmentName)){
        	// set the response as output Source
        	Source source = createSourceFromOutputStream(outContent);
        	exchange.setOutMessageContent(source);
        }else{
        	//set the response as attachment
        	DataHandler attachment =createDataSourceFromOutputStream(outContent, outputAttachmentName);
        	HashMap<String, DataHandler> attachments = new HashMap<String, DataHandler>();
        	attachments.put(outputAttachmentName,attachment);
        	exchange.setOutMessageAttachements(attachments);
        	String outSource = "<attached-files><file-name>"+outputAttachmentName+"</file-name></attached-files>";
        	exchange.setOutMessageContent(outSource);
        }
        
    }
    
    /**
     * Retrieve the InputStream associated to the desired xsl
     * from the su classloader or directory
     * @param exchange
     * @param extensions
     * @return
     */
    private InputStream getXslInputStream(Exchange exchange,
            Extensions extensions) {
    	getLogger().log(Level.INFO,"Retrieve the XSL style sheet to use.");

        String suRootPath = ((ServiceUnitManager) this.getComponent()
                .getServiceUnitManager())
                .getSUInstallRootForActivatedEp(exchange.getEndpoint());

        String xslName = extensions
        .getPetalsExtensions().getProperty(JBIListener.XSL);

        // Try to retrieve the XSL from the classloader (XSL is in a JAR file)
        InputStream xsl = getXslResourceInputStream(exchange, xslName, suRootPath);
        if(xsl == null){
        	xsl = getXslFileInputStream(exchange, xslName, suRootPath);
        }
        return xsl;
    }

/**
 * We retrieve the InputStream associated to the desired xsl
 * from the su classloader
 * @param exchange
 * @param xslName
 * @param suRootPath
 * @return the xsl inputStream, NULL if the resource does not exist
 */    private InputStream getXslResourceInputStream(Exchange exchange,
            String xslName, String suRootPath) {

    	ClassLoader classLoader = ClassLoaderUtil.createClassLoader(suRootPath,
                this.getClass().getClassLoader());
        
        return classLoader.getResourceAsStream(xslName);
    }
 
 /**
  * We retrieve the InputStream associated to the desired xsl
  * from the su directory
  * @param exchange
  * @param xslName
  * @param suRootPath
  * @return the xsl inputStream, NULL if the file does not exist
  */    
 private InputStream getXslFileInputStream(Exchange exchange,
             String xslName, String suRootPath) {

     	FileInputStream fileIS=null;
		try {
			fileIS = new FileInputStream(suRootPath+File.separator+xslName);
		} catch (FileNotFoundException e) {
			// nothing, there is no XSL file in the directory
		}
         return fileIS;
     }


    
    /**
     * Transform the Outputstream as a Source
     * @param outputStream
     * @return
     */
    private StreamSource createSourceFromOutputStream(
			ByteArrayOutputStream outputStream) {
    	getLogger().log(Level.INFO,"Transform the result as Source.");

		StreamSource sourceResult = new StreamSource();
		ByteArrayInputStream result = new ByteArrayInputStream(outputStream
				.toByteArray());
		sourceResult.setInputStream(result);
		return sourceResult;
	}

/**
 * Transform the outputstream as an IN DataHandler
 * @param outputStream
 * @param attachmentName
 * @return
 */
    private DataHandler createDataSourceFromOutputStream(
			ByteArrayOutputStream outputStream, String attachmentName) {
    	getLogger().log(Level.INFO,"Transform the result as an attachment.");
		DataHandler result;
		ByteArrayInputStream input = new ByteArrayInputStream(outputStream
				.toByteArray());
		StreamDataSource datasource = new StreamDataSource(input, attachmentName);
		result = new DataHandler(datasource);

	return result;
}
}
